if test -z "$CONFIG_SHELL"
then
    echo ""
    echo "--------------------------------------------------------------------------"
    echo "WARNING: you are running config/configure.sh without setting CONFIG_SHELL;"
    echo "this will work only if your current shell is bash or ksh."
    echo "--------------------------------------------------------------------------"
    echo ""
fi

src_root=".."

LOG="configure.log"

scripts="tcl php python lua ruby ghli perl stutter cli guile"

. lib/log.sh
. lib/fstools.lib
. lib/compile.lib
. lib/db.lib
. lib/subst.lib
. lib/test-awk.sh

# Databases:
#		DB_fstools        substitution database for fs tools (key: tool)
#		DB_includes       script module cc include flags (key: language)
#		DB_libs           script module cc lib flags (key: language)
#		DB_osdep          os dependent settings
#		DB_install_paths  installation paths

DB_libs=""
DB_includes=""

#----------------------------- Config part -------------------------------

. ./configure.defaults
. data/configure_params.tbl

function process_params {
for n in "$@"
do
	case "$n" in
		--help)
			$CONFIG_SHELL help.sh
			exit
			;;
		--include=*)
			value=`awk 'BEGIN { FS="=";  $0="'$n'"; print $2;}'`
			echo "including $value"
			echo "[cmdline] including $value" >> $LOG
			. $value
			;;
		--with*=*)
			name=`awk 'BEGIN { FS="="; $0="'$n'"; gsub("^--", "", $1); gsub("-", "_", $1); print $1;}'`
			value=`awk 'BEGIN { FS="=";  $0="'$n'"; print $2;}'`
			echo "[cmdline] --with: '$name' = '$value'" >> $LOG
			export $name=$value
			;;
		--without*)
			name=`awk 'BEGIN { FS="="; $0="'$n'"; gsub("^--without", "with", $1); gsub("-", "_", $1); print $1;}'`
			echo "[cmdline] --without: '$name'" >> $LOG
			export $name=""
			;;
		*)
			name=`echo $n | sed -e 's;^--;;' -e 's;=.*;;'`
			value=`awk 'BEGIN { FS="=";  $0="'$n'"; print $2;}'`
			echo name=$name
			echo value=$value
			real_name=`db_get "$name" configure_params`
			if test -z "$real_name"
			then
				echo "Uknown parameter: $n"
				echo "[cmdline] Uknown parameter: $n" >> $LOG
				exit 1
			else
				export $real_name=$value
			fi
			;;
	esac
done
}

#----------------------------- Paths -----------------------------------------
function set_install_paths {
	# if install paths are relative, insert install_prefix
	for n in with_install_path_doc with_install_path_man with_install_path_bin with_install_path_lib with_install_path_include
	do
		eval tmpn=\$$n
		case $tmpn in
			/*) export $n;;
			*) export $n="${with_install_prefix}/$tmpn" ;;
		esac
	done

	# These paths can't be changed from settings yet
	export with_install_path_lib_gpmi=$with_install_path_lib/gpmi
	export with_install_path_include_gpmi=$with_install_path_include/gpmi
	export with_install_path_pkg_lib=$with_install_path_lib/gpmi/gpmi_pkg
	export with_install_path_mod_lib=$with_install_path_lib/gpmi/modules
	export with_install_path_pkg_include=$with_install_path_include/gpmi_pkg
	export with_install_path_user_pkg_include=$with_install_path_user_prefix/gpmi_pkg
	export with_install_path_user_pkg_lib=$with_install_path_user_prefix/gpmi_pkg
	export with_install_path_user_mod_lib=$with_install_path_user_prefix/modules

	# Add all paths to DB_install_paths
	for n in install_path_doc install_path_man install_path_bin install_path_lib install_path_include install_path_lib_gpmi install_path_include_gpmi install_path_pkg_lib install_path_mod_lib install_path_pkg_include install_prefix install_path_user_pkg_include install_path_user_pkg_lib install_path_user_mod_lib
	do
		m=with_$n
		eval tmpm=\$$m
		db_add $n "$tmpm" DB_install_paths
	done
}

#----------------------------- Detect os tools -------------------------------
function os_detect {
	if test -z "$with_target"
	then
		if ! test -z "$with_target_dir"
		then
			export with_target="$with_target_dir-"
		else
			export with_target=""
		fi
	fi
	if test -z "$with_host"
	then
		if ! test -z "$with_host_dir"
		then
			export with_host="$with_host_dir-"
		else
			export with_host=""
		fi
	fi

	# If host and target don't match, we can safely assume it is a cross-compile
	if ! test "$with_host" = "$with_target"
	then
		with_cross="true"
	else
		with_cross="false"
	fi
	export with_cross
	
	if test -z "$with_os"
	then
		log_start "OS" "Autodetecting os... "
		with_os=`$CONFIG_SHELL lib/os-detect.sh`
		if test -z "$with_os"
		then
			echo "FAILED."
			echo ""
			echo "Critical: can't autodetect os."
			echo "[OS] Critical: can't autodetect os." >> $LOG
			exit
		fi
		log_end "OS" "found '$with_os'"
	else
		log_end "OS" "Using user supplied os param: '$with_os'"
	fi

	export with_sys_usrcustom

	log_start "OS" "Configuring platform-dependent settings..."
	if ! test -f "os/${with_os}.sh"
	then
		echo "FAILED"
		echo ""
		echo "Critical: os/${with_os}.sh is not found, so I don't know how to handle operating system ${with_os}."
		echo "[OS] Critical: os/${with_os}.sh is not found, so I don't know how to handle operating system ${with_os}." >> $LOG
		exit
	fi

	echo " [OS] including os/${with_os}.sh" >> $LOG
	. os/${with_os}.sh

	if test "$?" != "0"
	then
		echo "FAILED"
		echo ""
		echo "Critical: os/${with_os}.sh exited with error status $?."
		echo "[OS] Critical: os/${with_os}.sh exited with error status $?." >> $LOG
		exit
	fi
	log_end "OS" "Done."

	log_start "OS" "Setting source path..."
	export with_gpmi_src_path=`cd ..; pwd`
	log_end "OS" "$with_gpmi_src_path"

	log_start "OS" "Setting up platform dependency database..."
	#set up os dependent DB
	db_add sys_cross "$with_cross" DB_osdep
	db_add sys_cflags "$with_sys_cflags" DB_osdep
	db_add sys_ldflags "$with_sys_ldfags" DB_osdep
	db_add sys_ldflags_implib "$with_sys_ldfags_implib" DB_osdep
	db_add sys_ldflags_static "$with_sys_ldflags_static" DB_osdep
	db_add sys_ldflags_dynamic "$with_sys_ldflags_dynamic" DB_osdep
	db_add sys_libext_static "$with_sys_libext_static" DB_osdep
	db_add sys_libext_dynamic "$with_sys_libext_dynamic" DB_osdep
	db_add sys_libext_lib_dynamic "$with_sys_libext_lib_dynamic" DB_osdep
	db_add sys_libext_exe "$with_sys_libext_exe" DB_osdep
	db_add sys_dyn_0 "$with_sys_dyn_0" DB_osdep
	db_add sys_dyn_imp "$with_sys_dyn_imp" DB_osdep
	db_add sys_cc "$with_cc" DB_osdep
	db_add sys_cc_host "$with_cc_host" DB_osdep
	db_add sys_ld "$with_ld" DB_osdep
	db_add sys_strip "$with_strip" DB_osdep
	db_add sys_ldl "$with_sys_ldl" DB_osdep
	db_add sys_fpic "$with_sys_fpic" DB_osdep
	db_add sys_rdynamic "$with_sys_rdynamic" DB_osdep
	db_add sys_usrcustom "$with_sys_usrcustom" DB_osdep
	db_add sys_archive_objs "$with_sys_archive_objs" DB_osdep
	db_add with_os "$with_os" DB_osdep
	db_add target_dir "$with_target_dir" DB_osdep
	db_add target "$with_target" DB_osdep
	db_add host_dir "$with_host_dir" DB_osdep
	db_add host "$with_host" DB_osdep
	db_add gpmi_src_path "$with_gpmi_src_path" DB_osdep
	
	# Export some variables that might be needed by other detect scripts
	export with_sys_ldl
	export with_sys_fpic
	export with_sys_ld
	export with_sys_strip
	export with_cc
	export with_cc_host
	export with_os
	export with_sys_libext_exe
	export with_sys_cflags
	export with_sys_ldflags
	export with_sys_emulator
	export with_sys_calls

	log_end "OS" "Done."
}

#----------------------------- Detect fs tools -------------------------------
function fs_test {
	log_start "FS" "fstools: detecting '$1'... "

	eval fs_test_p2=\$$2
	if test -z "$fs_test_p2"
	then
		tester=$3
		name=$2
		array=$1
		shift 3
		for n in "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9"
		do
			if ! test -z "$n"
			then
				$tester $n
				if test "$?" = 0
				then
					db_add "$array" "$n" DB_fstools
					echo "export with_fs_${array}=\"${n}\"" >> Install-fstools.sh
					export $name="$n"
					log_end "FS" "(\"$n\", detected)"
					case $n in
						*unsafe*)
							echo ""
							echo "************************ WARNING **************************"
							echo "The only working $1 is $n, which is unsafe. Please check $n and whether you trust it."
							echo ""
							echo "Do you want to continue? (please type \"yes\" if you are sure)"
							echo ""
							read cont
							if ! test "$cont" = "yes"
							then
								echo "Critical: user didn't accept $n for $1."
								echo "[FS] Critical: user didn't accept $n for $1." >> $LOG
								exit 1
							fi
						;;
					esac							
					return
				fi
			fi
		done
		echo "FAILED."
		echo ""
		echo "Critical: Can't find a working $1."
		echo "[FS] Critical: Can't find a working $1." >> $LOG
		exit 1
	else
		$3 $fs_test_p2

		if test "$?" = 0
		then
			db_add "$1" "$fs_test_p2" DB_fstools
			export $2
			echo "export with_fs_${1}=\"$fs_test_p2\"" >> Install-fstools.sh
			echo "ok. (\"$fs_test_p2\", user supplied)"
			echo "[FS] \"$fs_test_p2\", user supplied" >> $LOG
		else
			echo "FAILED."
			echo ""
			echo "Critical: User supplied $1 (\"$fs_test_p2\") failed."
			echo "[FS] Critical: User supplied $1 (\"$fs_test_p2\") failed." >> $LOG
			exit 1
		fi
	fi

}

function fs_detect {
	# IMPORTANT: order of these tests DOES matter!
	echo "[FS] fs_detect() start" >> $LOG

	fs_test	"tempfile" "with_fs_tempfile" fstest_tempfile "tempfile" "lib/tempfile.sh" "lib/tempfile_unsafe.sh"
	fs_test "cat" "with_fs_cat" fstest_cat "cat" "type"
	fs_test "head" "with_fs_head" fstest_head "head" "lib/head.sh"
	fs_test "mkdir" "with_fs_mkdir" fstest_mkdir "mkdir -p"
	fs_test "cp" "with_fs_cp" fstest_copy "cp -rpH" "cp -r"
	fs_test "mv" "with_fs_mv" fstest_move "mv"
	fs_test "rm" "with_fs_rm" fstest_remove "rm -rf"
	fs_test "ln" "with_fs_ln" fstest_link "ln -s" "ln" "cp"
	fs_test "awk" "with_fs_awk" test_awk "awk" "gawk" "mawk"
	fs_test "find" "with_fs_find" fstest_find "find" "/usr/bin/find" "lib/find.sh"
	fs_test "ar" "with_fs_ar" fstest_ar "${with_target}ar" "ar" "/usr/bin/ar"
	fs_test "ranlib" "with_fs_ranlib" fstest_ranlib "${with_target}ranlib" "ranlib" "/usr/bin/ranlib"

	echo "[FS] fs_detect() finish" >> $LOG
}

#----------------------------- Detect compiler tools -------------------------------

function reset_dlc
{
		db_add sys_dlc_ld "" DB_osdep
}

# Detect dlc trying a speficif dir
function dlc_detect_ {
	# For DLC target, check if we can find DLC
	cfg=`$CONFIG_SHELL lib/dlc-config.sh $1 | $with_fs_head -n 1`
	if test -z "$cfg"
	then
		return 1
	fi

	dlc_detected="1"
	dlc_includes=`echo "$cfg" | awk -v "FS=|" '{print $1}'`
	dlc_libs=`echo "$cfg" | awk -v "FS=|" '{print $2}'`
	dlc_prefix=`echo "$cfg" | awk -v "FS=|" '{print $3}'`
	if ! test -z "$dlc_prefix"
	then
		dlc_prefix="${dlc_prefix}/bin/"
	fi

	with_sys_cflags="${dlc_includes} ${with_sys_cflags}"
	with_sys_ldl="${dlc_libs}"

	db_add sys_cflags "$with_sys_cflags" DB_osdep
	db_add sys_ldl "$with_sys_ldl" DB_osdep
	db_add sys_dlc_ld "${dlc_prefix}dlc-ld" DB_osdep

	dlc_ld="${dlc_prefix}dlc-ld"
	export dlc_ld
	export dlc_detected

	return 0;

}

# Detect dlc trying a list of dirs
function dlc_detect
{
	reset_dlc
	for n in "$@"
	do
		dlc_detect_ $n && return 0
	done
	return 1
}

function compiler_detect {
	if ! test -z "$with_target_dir"
	then
		log_end "CC" "Forced target system: '$with_target_dir'"
	fi
	if ! test -z "$with_host_dir"
	then
		log_end "CC" "Forced host system: '$with_host_dir'"
	fi


	log_start "CC" "Detecting c compiler... "

	CFLAGS="$with_cc_O $with_cc_debug"
	if test -z "$with_cc"
	then
		detect_compiler
		if test "$?" != 0
		then
			echo "FAILED."
			echo "Critical: can't find a working c compiler"
			echo "[CC] Critical: can't find a working c compiler" >> $LOG
			exit 1
		fi
		export with_cc=$CC
		db_add sys_cc "$with_cc" DB_osdep
		log_end "CC" "(\"$CC $CFLAGS\", detected)"
	else
		CC=$with_cc
		check_compiler
		if test "$?" != 0
		then
			log_end "CC" "Critical: c compiler failed (\"$with_cc $CFLAGS\", user supplied)"
			exit 1
		fi
		log_end "CC" "ok. (\"$CC $CFLAGS\", user supplied)"
	fi

	if test -z "$with_cc_host"
	then
		if ! test -z "$with_host_dir"
		then
			export with_cc_host="${with_host}gcc"
		else
			if test "$with_cross" == "true"
			then
				export with_cc_host=cc
			else
				export with_cc_host=$with_cc
			fi
		fi
	fi
	db_add sys_cc_host "$with_cc_host" DB_osdep

	export with_cc CC CFLAGS

	if test "$with_os" = "windlc"
	then
		if test -z "$with_dlc_prefix"
		then
			log_start "DLC" "Detecting DL-Compat... "
			dlc_detect "/usr/local" "/usr"
		else
			log_start "DLC" "Testing DL-Compat ($with_dlc_prefix)... "
			dlc_detect "$with_dlc_prefix"
		fi

		if test -z "$dlc_detected"
		then
			echo "[DLC] FAILED." >> $LOG
			echo "FAILED."
			echo ""
			echo "DL-Compat couldn't be found. Please install it before compiling GPMI."
			echo "DL-Compat can be found at http://www.libgpmi.org."
			exit 1
		else
			if test -z "${dlc_prefix}"
			then
				log_end "DLC" "found global"
			else
				dlc_dirname=`dirname $dlc_prefix`
				log_end "DLC" "found '$dlc_dirname'"
			fi
		fi
	else
		reset_dlc
	fi

	. lib/soname.sh

	if test -z "$with_sys_makedep"
	then
		log_start "MAKE" "Detecting make \$^... "
		with_sys_makedep=`$CONFIG_SHELL lib/detect_make_dep.sh`
	else
		log_start "MAKE" "Using user supplied make \$^: $with_sys_makedep "
	fi
	if test -z "$with_sys_makedep"
	then
		echo ""
		echo "ERROR:"
		echo "User supplied makedep (make \$^) is empty and I couldn't find a working one."
		echo "You may want to install GNU make."
		exit 1
	fi	
	log_end "MAKE" "OK ($with_sys_makedep)"

	export with_sys_makedep
	db_add sys_makedep "$with_sys_makedep" DB_osdep
}

#----------------------------- Detect standard calls ---------------------------
function stdcall_detect_
{
	# $1: variable name
	# $2: db name
	# $3: script
	eval tmp1=\$$1
	if test -z "$tmp1"
	then
		tmp="`$CONFIG_SHELL $3`"
		if test "$?" != 0
		then
			log_end "STD" "Critical: unknown $1 output ($tmp) "
			exit 1
		fi
		export $1="$tmp"
		eval tmp1=\$$1
		log_end "STD" "OK; detected $tmp1"
	else
		log_end "STD" "Using user supplied $tmp1"
	fi
	db_add $2 "$tmp1" DB_osdep
}

function stdcall_detect {
	log_start "STD" "Detecting printf %%p prefix..."
	stdcall_detect_ with_std_printfp std_printfp lib/detect_printf.sh

	log_start "STD" "Detecting printf %%x cast..."
	stdcall_detect_ with_std_printfp_cast std_printfp_cast lib/detect_printf_cast.sh

	log_start "STD" "Detecting snprintf..."
	stdcall_detect_ with_std_snprintf std_snprintf lib/detect_snprintf.sh

	log_start "STD" "Detecting vararg macros..."
	stdcall_detect_ with_std_varargmacro std_varargmacro lib/detect_varargmacro.sh

	log_start "STD" "Detecting inline..."
	stdcall_detect_ with_std_inline std_inline lib/detect_inline.sh

	log_start "STD" "Detecting __FUNCTION__..."
	stdcall_detect_ with_std_function std_function lib/detect_function.sh

	log_start "STD" "Detecting constructor..."
	stdcall_detect_ with_std_constructor std_constructor lib/detect_constructor.sh

	if test -z "$with_std_constructor"
	then
		log_start "STD" "trying -init ldflag instead..."
		stdcall_detect_ with_sys_init sys_init lib/detect_ld_init.sh
	fi

	log_start "STD" "Detecting destructor... "
	stdcall_detect_ with_std_destructor std_destructor lib/detect_destructor.sh
	
	db_add sys_calls "" DB_osdep
	db_add sys_init  "$with_sys_init" DB_osdep
}


#----------------------------- Detect scripts tools -------------------------------
function test_script {
	tmp="with_$1"
	tmpp1="with_$1_params_libs"
	tmpp2="with_$2_params_includes"
	eval tmptmp=\$$tmp
	eval tmptmpp1=\$$tmpp1
	eval tmptmpp2=\$$tmpp2

	if test "$tmptmp" = "1"
	then
		if test -z "$tmptmpp1$tmptmpp2"
		then
			log_start "SCRIPT" "Scripts: trying to configure $1..."
			if test -f "lib/$1-config.sh"
			then
				cfg=`$CONFIG_SHELL lib/$1-config.sh | $with_fs_head -n 1`
				if test -z "$cfg"
				then
					log_end "SCRIPT" "Failed: not found. Deselected."
					export with_$1=0
				else
					log_end "SCRIPT" "ok (params: $cfg)"
					includes=`echo "$cfg" | awk -v "FS=|" '{print $1}'`
					libs=`echo "$cfg" | awk -v "FS=|" '{print $2}'`
					db_add "$1" "$includes" DB_includes
					db_add "$1" "$libs" DB_libs
					export scripts_found="$scripts_found $1"
				fi
			else
				log_end "SCRIPT" "FAILED. (Can't find the config script). Deselected."
				export with_$1=0
			fi
		else
			log_start "SCRIPT" "using user supplied configuration for $1: '$tmpp1' '$tmpp2'" 
			db_add "$1" "$tmptmpp1" DB_libs
			db_add "$1" "$tmptmpp2" DB_includes
			export scripts_found="$scripts_found $1"
			log_end "SCRIPT" "Done."
		fi
	else
		log_end "SCRIPT" "$1 is not selected, skipping."
	fi
}

function script_detect {
	export scripts_found=""
	echo "[SCRIPT] script_detect() start" >> $LOG
	for n in $scripts
	do
		test_script $n
	done
	echo "[SCRIPT] script_detect() finish" >> $LOG
}

#----------------------------- Create makefiles -------------------------------
function makefile_create {
	echo "[MAKE] makefile_create start" >> $LOG

	# NOTE: I use the following formula for each generation:
	#   "cat input_file | subst DB | subst DB | ... | subst DB > output_file"
	# Instead of the initial cat, I could redirect the input file into the
	# first subst, but I believe this way it is all more readable.

	log_start "MAKE" "creating makefiles for core packages..."
	for n in `$CONFIG_SHELL ./gpmi-config.tmp --makevar PACKAGES < $src_root/packages/Makefile.am`
	do
		$with_fs_cat data/Makefile.core-pkg | subst DB_fstools > $src_root/packages/$n/Makefile
		mkdir -p $src_root/packages/$n/generated
	done
	log_end "MAKE" "Done."

	with_example=1

	log_start "MAKE" "creating makefiles for core modules..."

	export DB_includes
	export DB_libs

	for n in `$CONFIG_SHELL ./gpmi-config.tmp --makevar MODULES < $src_root/modules/Makefile.am`
	do
		flag=with_$n
		eval tmpflag=\$$flag
		if test "$tmpflag" = "1"
		then
			tmp=""
			a1=`db_get "$n" DB_includes`
			db_add modcflags "$a1" tmp
			a2=`db_get "$n" DB_libs`
			db_add modldflags "$a2" tmp
			$with_fs_cat data/Makefile.core-mod  | subst tmp | subst DB_fstools > $src_root/modules/$n/Makefile
		else
			cp data/Makefile.empty $src_root/modules/$n/Makefile
		fi
	done
	echo "SCRIPTS_CONFIGURED =$scripts_found" > $src_root/modules/Makefile.configured
	log_end "MAKE" "Done."

	log_start "MAKE" "creating makefiles for example apps..."
	for n in `$CONFIG_SHELL ./gpmi-config.tmp --makevar EXAMPLES_gen < $src_root/example-app/Makefile.am`
	do
		$with_fs_cat data/Makefile.examples | subst DB_fstools > $src_root/example-app/$n/Makefile
	done
	log_end "MAKE" "Done."

	log_start "MAKE" "creating Makefile.config..."
	$with_fs_cat data/Makefile.config | subst DB_fstools | subst DB_osdep | subst DB_install_paths > $src_root/Makefile.config
	log_end "MAKE" "Done."

	echo "[MAKE] makefile_create finish" >> $LOG
}

#----------------------------- Create header files --------------------------
function header_create {
	echo "[H] header_create start" >> $LOG

	log_start "H" "creating gpmi_config.h"
	$with_fs_cat data/config.h | subst DB_osdep > $src_root/gpmi_config.h
	log_end "H" "Done."

	echo "[H] header_create finish" >> $LOG
}

#--------------------- Create script test's shell script ---------------------------

function test_sh_create {
	echo "[TEST] test_sh_create start" >> $LOG

$with_fs_cp data/test.sh $src_root/tests/app-scripts/test.sh
	echo "[TEST]  scripts: '$scripts' " >> $LOG
for n in $scripts
do
	tmp=with_$n
	eval tmptmp=\$$tmp
	if test "$tmptmp" = "1"
	then
	echo "[TEST]  generating test.sh for: '$n' " >> $LOG
	msg=`echo "Testing $n-module... " | awk '{tmp=$0; for(n=length(tmp); n<40; n++) { tmp=tmp " "} print tmp }'`
	echo "printf \"$msg\"" >> $src_root/tests/app-scripts/test.sh

	test_file="script"

	# XXX: this should be replaced; this info should be data, not code.
	# cli has another output because forking
	[ $n = "cli" ] && test_file="cli"

	echo "[TEST]  test_file: '$test_file' " >> $LOG

	echo '

# make sure we will not use an old out
'$with_fs_rm' out
$WRAPPER $EMULATOR $EXE '$n' example_'$n' > out 2> /dev/null
../test.awk '$test_file'.OK < out || fail=1
'$with_fs_rm' out

	' >> $src_root/tests/app-scripts/test.sh
	else
		echo "# Not testing $n as it is deselected" >> $src_root/tests/app-scripts/test.sh
	fi
done

	echo '
if ! test -z "$fail"
then
	echo "FAILED (one or more script module(s) has/have failed)"
	false
else
	true
fi
	'>> $src_root/tests/app-scripts/test.sh

	echo "[TEST] test_sh_create finish" >> $LOG
}

function install_create {
	echo "[INSTALL] install_create start" >> $LOG

	log_start "INSTALL" "Creating extra install variables..."
	echo "export with_os=$with_os" >> Install-fstools.sh
	echo "export with_sys_libext_dynamic=$with_sys_libext_dynamic" >> Install-fstools.sh
	echo "export with_sys_libext_lib_dynamic=$with_sys_libext_lib_dynamic" >> Install-fstools.sh
	echo "export with_sys_libext_exe=$with_sys_libext_exe" >> Install-fstools.sh
	echo "" >> Install-fstools.sh
	echo "# Installation paths " >> Install-fstools.sh
	echo "$DB_install_paths "| awk -v "FS=>" '{print "export gpmi_" $1 "=" $2}' >> Install-fstools.sh
	log_end "INSTALL" "Done."

	log_start "INSTALL" "Creating gpmi-config"
	tmp=""
	db_add vars "$with_install_path_lib_gpmi/Install-fstools.sh" tmp
	db_add with_fs_awk "$with_fs_awk" tmp
	db_add with_fs_rm "$with_fs_rm" tmp
	db_add with_fs_cp "$with_fs_cp" tmp
	db_add sys_init "$with_sys_init" tmp

	db_add listgen "host.listgen${with_sys_libext_exe}" tmp
	db_add wrapgen "host.wrapgen${with_sys_libext_exe}" tmp
	if ! test -z "$dlc_includes"
	then
		db_add includes "${dlc_includes}" tmp
	else
		db_add includes "" tmp
	fi

	# Derive the sed script that can replace ### fields of fstools and install_paths 
	replace=`echo "$DB_install_paths
$DB_fstools" | $with_fs_awk -v "FS=>" '
	{
		if (/^install_path/) {$1="gpmi_" $1} else {$1="with_fs_" $1}
		printf("	s@###%s###@$%s@g;###newline###", $1, $1)
	}
	'`

	db_add replace_install_paths "$replace" tmp
	if test -z "$CONFIG_SHELL"
	then
	    echo "#!/bin/bash" > gpmi-config
	else
    	    echo "#!$CONFIG_SHELL" > gpmi-config
	fi
	subst tmp < data/gpmi-config | sed "s/###newline###//g" | tr "" "\n" >> gpmi-config
	chmod 0755 gpmi-config
	log_end "INSTALL" "Done."
	echo "[INSTALL] install_create finish" >> $LOG
}

function makefile_vars {
	with_sys_cflags="${with_cc_O}   ${with_cc_debug}   ${with_sys_cflags}"
	db_add sys_cflags "$with_sys_cflags" DB_osdep
	db_add sys_ldflags "$with_sys_ldflags" DB_osdep
}

function set_options_ {
	target_var="with_sys_$opt"
	target_opt="options_$opt"
	eval tmptarget_var=\$$target_var
	eval tmptarget_opt=\$$target_opt
	echo "[OPTIONS] setting $target_var; before: $tmptarget_var" >> $LOG
	for n in $tmptarget_opt
	do
		name=${n%%\>*}
		value=${n#*\>}
		eval tmpname=\$$name
		echo "[OPTIONS]   $name ($tmpname) = $value" >> $LOG
		if test "$tmpname" = 1
		then
			export $target_var="$tmptarget_var $value"
		fi
	done
	echo "[OPTIONS] setting $target_var;  after: $tmptarget_var" >> $LOG
}

function set_options {
	log_start "OPTS" "Setting up options..."

	# Include options here so variables are evaluated
	. data/configure_options.tbl

	for opt in $option_tables
	do
		log_start "" "[$opt] "
		set_options_
	done

	log_end "OPTS" "done."
}


function touch_files {
	touch ../wrapgen/listgen1/listgen.tab.* ../wrapgen/listgen1/lex.*
}

function util_scripts_create {
	subst DB_osdep < data/gpmi_src_wrap.sh > ../gpmi_src_wrap.sh
	chmod 755 ../gpmi_src_wrap.sh
}

function gpmi_config_tmp {
	sed "s@###vars###@./Install-fstools.sh@g" < data/gpmi-config > gpmi-config.tmp
	chmod 755 gpmi-config.tmp
}

#----------------------------- Main -----------------------------------------

export LOG
echo "[START]" > $LOG
date >> $LOG

if test -f "local.conf"
then
	echo "[CONF] Found local conf in local.conf, including" >> $LOG
	. ./local.conf
fi

echo "[cmdline] Parsing command line arguments..." >> $LOG
# Set up base configuration
process_params "$@"



echo "Configuring gpmi..."
echo ""

echo "">Install-fstools.sh

set_install_paths


# Detect operating system
os_detect

# Detect fs tools
fs_detect

# Detect compiler
compiler_detect

# Detect libs
if test "$with_option_threads" = "1"
then
	. lib/pthreads-config.sh
fi

# Detect standard calls
stdcall_detect

# Set options
set_options

# Detect script modules
script_detect

# Set up gpmi-config.tmp
gpmi_config_tmp

# Set up makefile variables
makefile_vars

# Create makefiles
makefile_create

# Create headers
header_create

# Create test.sh for scripts
test_sh_create

# Create utility scripts
util_scripts_create

# Create extra install variables
install_create

# Touch some files to make sure they are not regenerated after a fresh checkout
touch_files
