/*
    libgpmi - General Package/Module Interface
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef GPMI_LIBS_PACKAGE_H
#define GPMI_LIBS_PACKAGE_H

/* The gpmi_package struct */
struct gpmi_package {
	gpmi_library *library;                      //! Underlying library

	char **name_space;                          //! name space of a package - used to resolve other routines

	gpmi_PkgProcInit     *proc_Init;            //! Proc to initialize function   (Optional)
	gpmi_PkgProcUnInit   *proc_UnInit;          //! Proc to uninitialize function (Optional)
	gpmi_PkgProcCheckVer *proc_CheckVer;        //! Proc to check version         (Optional)
	gpmi_PkgWProcRegisterCommands	*proc_RegCmd; //! Proc to start command registering

	gpmi_package         *next;                 //! Next package - used for creating linked lists
};

/* Functions available for public */
int gpmi_pkg_load(const char *library_name, int required_version, gpmi_PkgProcRegisterCommand *regcmd, void *handle, gpmi_package **pkg_handle);
int gpmi_pkg_load_lib(gpmi_library *library, int required_version, gpmi_PkgProcRegisterCommand *regcmd, void *handle, gpmi_package **pkg_handle);
void gpmi_pkg_unload(gpmi_package *package);
void *gpmi_pkg_resolve(gpmi_package *package, const char *name);

/* Low level functions */
gpmi_package *gpmi_pkg_alloc(gpmi_library *library);
void gpmi_pkg_free(gpmi_package *package);

/* Internal functions */
char *gpmi_pkg_misc_func(char *name_space, int name_space_len, char *func, int func_len);

#endif /* GPMI_LIBS_PACKAGE_H */
