/*
    libgpmi - General Package/Module Interface
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef GPMI_LIBS_H
#define GPMI_LIBS_H

#include <gpmi/global.h>
#define OS_DEP_MACROS_ONLY
#include <gpmi/os_dep.h>
#undef OS_DEP_MACROS_ONLY

/* Module procedures */
typedef int  gpmi_ProcInit   (gpmi_module *module, const char *params);
typedef void gpmi_ProcUnInit (gpmi_module *module);
typedef void gpmi_ProcRunTick(gpmi_module *module, int frame_counter);
typedef void gpmi_ProcRunData(gpmi_module *module, void *data);
typedef int  gpmi_ProcDebug  (gpmi_module *module, const char *command);
typedef void gpmi_ProcEvent  (gpmi_module *module, int event, ...);

/* Package procedures */
typedef int  gpmi_PkgProcCheckVer(int version_we_need);
typedef void gpmi_PkgProcRegisterCommand(void *handle, char *name, void *proc_wrapper, void *proc_lowlevel, gpmi_package *pkg);
typedef void gpmi_PkgWProcRegisterCommands(void *callback, void *handle, gpmi_package *pkg);
typedef int  gpmi_PkgProcInit(void);
typedef void gpmi_PkgProcUnInit(void);

typedef enum gpmi_library_flags {
	gpmi_libf_loading = 1,                  //! set during the load of a library - used to detect dependency loops
#ifndef gpmi_no_static
	gpmi_libf_static  = 2,                  //! static library - linked in, do not use dlsym
#endif
} gpmi_library_flags_t;

typedef struct gpmi_library {
	char *name;                             //! Name of opened library
	gpmi_library_handle handle;             //! Handle(s) of current library (UNIX: null of no library)

	int references;                         //! Reference countunter; when drops to 0, lib is unloaded

	struct gpmi_library *next;              //! Next element in the double linked list of loaded libraries
	struct gpmi_library *prev;              //! Previous element in the double linked list of loaded libraries
	gpmi_library_flags_t flags;             //! See the following enum
	gpmi_package *static_package;           //! Static package structure if the library is a static package
	gpmi_package *libdep;                   //! Single linked list of packages loaded as dependency
} gpmi_library;


/* Low level functions */
char *gpmi_library_find_path(char *dir_list, const char *library_name, char *file_name, char *dep_file_name, int path_max);
gpmi_library *gpmi_load_library(char **dir_list, const char *library_name, int *state, int is_pkg);
void gpmi_unload_library(gpmi_library *library, void (*uninit)(void) );
gpmi_library *gpmi_find_library(const char *library_name);
gpmi_library *gpmi_lib_create(char *library_name);
int gpmi_libdep_load(gpmi_library *lib, const char *dep_file_name);
void gpmi_libdep_unload(gpmi_library *lib);

#endif /* GPMI_LIBS_H */
