/*
    libgpmi - General Package/Module Interface - bgproc package
    Copyright (C) 2006-2007 Tibor 'Igor2' Palinkas & glx 
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include "bgproc_os_dep.h"


int bgproc_create_pipe(pipe_t p[2], PipeType t)
{
#ifdef WIN32
	SECURITY_ATTRIBUTES saAttr;
	BOOL res;
	// Set the bInheritHandle flag so pipe handles are inherited.
	saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
	saAttr.bInheritHandle = TRUE;
	saAttr.lpSecurityDescriptor = NULL;
	res = CreatePipe(&p[BGPROC_PIPE_READ], &p[BGPROC_PIPE_WRITE], &saAttr, 0);
	// Ensure the (t) handle to the pipe is not inherited.
	if (res) {
		SetHandleInformation(p[t], HANDLE_FLAG_INHERIT, 0);
		return 0;
	}
	return -1;
#else
	int ret;

	ret = pipe(p);
	if (t == BGPROC_PIPE_READ) {
		fcntl(p[0], F_SETFL, O_NONBLOCK);
		fcntl(p[1], F_SETFL, O_NONBLOCK);
	}
	return ret;
#endif
}

void bgproc_close_pipe(pipe_t p[2], PipeType t)
{
#ifdef WIN32
	CloseHandle(p[t]);
#else
	close(p[t]);
#endif
}

int bgproc_create_child(const char *commandline, pipe_t pa[2], pipe_t pb[2])
{
#ifdef WIN32
	PROCESS_INFORMATION piProcInfo;
	STARTUPINFO siStartInfo;

// Set up members of the PROCESS_INFORMATION structure.
	ZeroMemory( &piProcInfo, sizeof(PROCESS_INFORMATION) );

// Set up members of the STARTUPINFO structure.
	ZeroMemory( &siStartInfo, sizeof(STARTUPINFO) );
	siStartInfo.cb = sizeof(STARTUPINFO);
	siStartInfo.hStdInput = pa[BGPROC_PIPE_READ];
	siStartInfo.hStdOutput = pb[BGPROC_PIPE_WRITE];
	siStartInfo.hStdError = GetStdHandle(STD_ERROR_HANDLE);
	siStartInfo.dwFlags |= STARTF_USESTDHANDLES;

// Create the child process.
	if (CreateProcess(NULL,
		commandline,   // command line
		NULL,          // process security attributes
		NULL,          // primary thread security attributes
		TRUE,          // handles are inherited
		0,             // creation flags
		NULL,          // use parent's environment
		NULL,          // use parent's current directory
		&siStartInfo,  // STARTUPINFO pointer
		&piProcInfo)  // receives PROCESS_INFORMATION
	== 0) {
		return -1;
	}

	_hProcess = piProcInfo.hProcess;
	CloseHandle(piProcInfo.hThread);
#else
	pid_t p;

	p = fork();
	if (p == 0) {
		/* Child process */
		close(pa[BGPROC_PIPE_WRITE]);
		close(pb[BGPROC_PIPE_READ]);
		dup2(pa[BGPROC_PIPE_READ], 0);
		dup2(pb[BGPROC_PIPE_WRITE], 1);
		{
			char **argv;
			const char *s, *next;
			int argc, n;

			argv = malloc(sizeof(char *) * 64);
			for(argc = 0, s = commandline; argc < 63; s = next, argc++) {
				next = strchr(s + 1, ' ');
				n = 0;
				if (next == NULL) {
					next = s + strlen(s) - 1;
					n++;
				}
				while (*s == ' ') {
					s++;
					n++;
				}
				argv[argc] = malloc(next - s  + 2 + n);
				strncpy(argv[argc], s, next - s + n);
				if (*(next+1) == '\0') {
					argc++;
					break;
				}
			}
			argv[argc] = NULL;
/*			for(n = 0; n <= argc; n++) {
				printf(" %d -> '%s'\n", n, argv[n]);
			}
			fflush(stdout);*/
			execv(argv[0], argv);
		}

		printf("bgproc: Someone set up us the bomb (%s): %d/%s!\n", commandline, errno, strerror(errno));
		return 1; /* never get here! */
	}
#endif

	bgproc_close_pipe(pa, BGPROC_PIPE_READ);
	bgproc_close_pipe(pb, BGPROC_PIPE_WRITE);
	return 0;
}


int bgproc_write_to_pipe(pipe_t p, const char *buf, int buf_len)
{
#ifdef WIN32
	DWORD len;
	WriteFile(p, buf, buf_len, &len, NULL);
	return len;
#else
	return write(p, buf, buf_len);
#endif
}

int bgproc_read_from_pipe(pipe_t p, char *buf, int buf_len)
{
#ifdef WIN32
	DWORD nread;
	int i;
	char *s_to   = buf;
	char *s_from = buf;

	PeekNamedPipe(p, NULL, 0, NULL, &nread, NULL); // find how many bytes are available

	if (nread == 0)
		return -1;

	ReadFile(p, buf, buf_len, &nread, NULL);

	/* remove '\r' added by windows ;) */
	for(i = 0; i < nread; i++, s_from++)
		if (*s_from != '\r')
			*(s_to++) = *s_from;

	return s_to - buf;
#else
	return read(p, buf, buf_len);
#endif
}

