/*
    libgpmi - General Package/Module Interface - listgen
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

%{
	#include "listgen.tab.h"
	#include "listgen1.h"

	static inline void eatup_count_lines()
	{
		char *s;
#ifdef LINENODEBUG
		fprintf(stderr, "LINE %d :: %s\n", yylglloc.first_line+1, yytext);
#endif
		for(s = yytext; *s != '\0'; s++) {
			if (*s == '\n') {
				yylglloc.first_column=1;
				yylglloc.first_line++;
			}
		}
	}
%}

DIGIT	[0-9]*
ID		[A-Za-z][A-Za-z0-9_]*
WS		[\t ]

%%
[/][/].*[\n]				{/* eat up comments */ yylglloc.first_line++; }
enum				{return ENUM;}
\/\*				{return CMT_OPEN;}
\*\/				{return CMT_CLOSE;}
unsigned		{return UNSIGNED;}
int					{return INT;}
double			{return DOUBLE;}
char				{return CHAR;}
void				{return VOID;}
event				{return EVENT;}
typedef			{return TYPEDEF;}
gpmi_keyword {return GPMI_KEYWORD;}
gpmi_fake_keyword {return GPMI_FAKE_KEYWORD;}

gpmi_define_event {return GPMI_DEFINE_EVENT;}
of					{return OF;}
multiple		{return MULTIPLE;}
dynamic			{return DYNAMIC;}
const				{return CONST;}
extern			{return EXTERN;}
union				{return UNION;}
direct			{return DIRECT;}
nowrap			{return NOWRAP;}
inline      {return INLINE;}
FAKETYPE		{return FAKETYPE;}


struct			{return STRUCT;}

{DIGIT}			{yylglval.num = atoi(yytext); return NUMBER;}
{ID}				{yylglval.str = strdup(yytext); return ID;}

[\r]
[\n]				{yylglloc.first_line++; yylglloc.first_column=1;}
[ \t]				{yylglloc.first_column++;}



^{WS}*#.*(\\\n.*)*		{/* Eat up some unhandled tokens */ eatup_count_lines();}

[*(){};:,=.<>!\[\]]	{return yytext[0];}

.						{verbprintf(3, "      (Warning: unrecognised: '%s' in line %d at %d)\n", yytext, yylglloc.first_line, yylglloc.first_column);}



%%
