/*
    libgpmi - General Package/Module Interface - str_buff
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <string.h>
#include "str_buff.h"

str_buff_t *str_buff_open()
{
	str_buff_t *b;

	b = malloc(sizeof(str_buff_t));

	b->buff      = NULL;
	b->allocated = 0;
	b->used      = 0;

	return b;
}

void str_buff_add(str_buff_t *b, char *s, int len)
{
	int u;

	u = b->used + len;

	if (u >= b->allocated) {
		b->allocated = u + 1024;
		b->buff = realloc(b->buff, b->allocated);
	}

	memcpy(b->buff + b->used, s, len);
	b->buff[u] = '\0';

	b->used = u;
}

void str_buff_print(FILE *f, str_buff_t *b)
{
	fprintf(f, "%s", b->buff);
}

void str_buff_close(str_buff_t *b)
{
	if (b->buff != NULL)
		free(b->buff);
	free(b);
}

#ifdef NO_VARARG_MACRO
#include <stdarg.h>

#ifndef STD_HAVE_SNPRINTFP
int vsnprintf(char *buf, int len, const char *format, va_list ap)
{
	return vsprintf(buf, format, ap);
}
#endif

void sbprintf(str_buff_t *str_buff, char *fmt, ...)
{
	va_list args;
	va_start(args, fmt);
	str_len_tmp = vsnprintf(str_buff_tmp, sizeof(str_buff_tmp), fmt, args);
	str_buff_add(str_buff, str_buff_tmp, str_len_tmp);
	va_end(args);
}
#endif
