/*
    libgpmi - General Package/Module Interface - wrapgen
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "wrapgen_parser.h"
#include "wrapgen_keywords.h"

char *type_names[]={
	"*KEYWORD*",
	"int",
	"string",
	"double",
	"void",
	"uint",
	"ptr",
	"uchar",
	"char",
	NULL
} ;


/* Split a line in two words */
void split(char *string, char *first, char *second)
{
	char *s;
	char *s1, *s2;

	/* Init */
	*first  = 0;
	*second = 0;

	/* Seek the first non-whitespace char */
	for(; *string == ' ' || *string == '\t'; string++);
	s1 = string;

	if ( *s1 == '\n' || *s1 == '\r' )
		return;

	/* Seek the first whitespace */
	for(s = string+1; *s != ' ' && *s != '\t' && *s != '\r' && *s != '\n' ; s++);
	*s = 0;

	/* Seek the first nonwhitespace */
	for(s++; *s == ' ' || *s == '\t' ; s++);
	s2 = s;

	/* Seek the first whitespace */
	for(s++; *s != ' ' && *s != '\t' && *s != '\n' && *s != '\r' && *s != 0; s++);
	*s = 0;

	strcpy(first, s1);
	strcpy(second, s2);
}


int FindType(char *name)
{
	char **s;
	int n;

	/* Linear search type names */
	for ( n = 0, s = type_names; *s != 0; s++, n++ ) {
		if ( strcasecmp(*s, name) == 0 )
			return n;
	}

	/* If not found, assume keyword */
	return type_keyword;
}

int GetTypeFlags(char *type, int line)
{
	char *s;
	int ret;

	if ( *type == 0 )
		return 0;

	ret = 0;
	s		= type;

	/* Leading non-alnums are treated as prefixes */
	while (!(isalnum(*s))) {
		switch (*s) {
			case '!': ret |= flag_dynamic;  break;
			case '@': ret |= flag_multiple; break;
			case '^': ret |= flag_const;    break;
			case '-': ret |= flag_direct;   break;
			default:
				fprintf(stderr, "Warning: invalid type prefix %c at line %d\n", *type, line);
		}
		s++;
	}

	/* Cut prefixes */
	if ( s != type)
		memmove(type, s, strlen(s)+2);

	return ret;
}

/* General wrapper generator - reads and parses a list */
void GenWrapper(FILE *input, void *generator, void *newproc, void *gotparam, void *fileoperations, const char *section)
{
	char s[256];
	char param1[256], param2[256];
	char *Param1, *Param2;
	char cmd[64], cmd_type[32];
	int line, cmd_type_flags, flags, type, number_of_params, in_action;
	char keyword[128], keyword_type[32];
	void *current;
	void *(*NewProc)();
	void (*Generator)(void *current, int line, char *cmd, char *cmd_type, int cmd_type_flags, int params);
	int (*GotParam)(void *current, int line, char *param_str, char *type_str, int flags, int type);
	void (*FileOperations)(int cmd);

	cmd_type_flags 		= 0;
	Generator      		= generator;
	NewProc        		= newproc;
	GotParam			 		= gotparam;
	FileOperations 		= fileoperations;
	*cmd           		= 0;
	line           		= 0;
	in_action			 		= 0;
	number_of_params	= 0;

	if (NewProc != NULL)
		current = NewProc();
	else
		current = NULL;

	if (FileOperations != NULL)
		FileOperations(open_files);

	/* Read each line */
	while (!(feof(input))) {
		line++;
		*s = 0;
		fgets(s, 254, input);

		if (in_action == 0) {
			/* Check section */
			if (strncmp(s,"$$$",3) == 0) {
				if (strncasecmp(s+3, section, strlen(section)) == 0)
					in_action = 1;
			}
		}
		else {
			/* Check if section ends */
			if (strncmp(s,"$$$",3) == 0)
				break;

			switch (*s) {
				/* Comment or empty string, we ignore this line */
				case '#':
				case ';':
					/* Create separator lines for comments */
					if (Generator != NULL)
						Generator(current, line, NULL, NULL, 0, 0);
				case '\n':
				case '\r':
				case 0:
					break;
				case '@':
					/* keyword definition; this is above sections */
					split(s+1, keyword, keyword_type);
					if (keyword_store(keyword, keyword_type) < 0)
						exit(1);
					break;

				/* Parameter */
				case ' ':
				case '\t':
					split(s, param1, param2);
					Param1 = param1;
					Param2 = param2;
					if ( *Param1 != 0 && *Param2 != 0) {
						flags = GetTypeFlags(Param2, line);
						type = FindType(Param2);
						number_of_params = GotParam(current, line, Param1, Param2, flags, type);
					}
					break;

				/* New routine */
				default:
				/* Generate the last routine if we had one */
					if (*cmd)
						Generator(current, line, cmd, cmd_type, cmd_type_flags, number_of_params);

					/* Parse the current routine name and type */
					split(s, cmd, cmd_type);
					cmd_type_flags 		= GetTypeFlags(cmd_type, line);

					if (NewProc != NULL)
						current        	= NewProc();

					number_of_params	= 0;
			}
		}	//switch *s
	}

	/* Do the last command */
	if (*cmd && Generator != NULL)
		Generator(current, line, cmd, cmd_type, cmd_type_flags, number_of_params);

	if (FileOperations != NULL)
		FileOperations(close_files);
}
